@@ -55,9 +55,9 @@ void mutt_buffer_free (BUFFER **p)
   if (!p || !*p)
     return;
 
-   FREE (&(*p)->data);
-   /* dptr is just an offset to data and shouldn't be freed */
-   FREE (p);		/* __FREE_CHECKED__ */
+  FREE (&(*p)->data);
+  /* dptr is just an offset to data and shouldn't be freed */
+  FREE (p);		/* __FREE_CHECKED__ */
 }
 
 void mutt_buffer_clear (BUFFER *b)
@@ -82,6 +82,11 @@ BUFFER *mutt_buffer_from (char *seed)
   return b;
 }
 
+size_t mutt_buffer_len (BUFFER *buf)
+{
+  return buf->dptr - buf->data;
+}
+
 /* Increases the allocated size of the buffer */
 void mutt_buffer_increase_size (BUFFER *buf, size_t new_size)
 {
@@ -172,7 +177,7 @@ int mutt_buffer_add_printf (BUFFER* buf, const char* fmt, ...)
 /* Dynamically grows a BUFFER to accommodate s, in increments of 128 bytes.
  * Always one byte bigger than necessary for the null terminator, and
  * the buffer is always null-terminated */
-static void mutt_buffer_add (BUFFER* buf, const char* s, size_t len)
+void mutt_buffer_addstr_n (BUFFER* buf, const char* s, size_t len)
 {
   if (buf->dptr + len + 1 > buf->data + buf->dsize)
     mutt_buffer_increase_size (buf, buf->dsize + (len < 128 ? 128 : len + 1));
@@ -183,12 +188,12 @@ static void mutt_buffer_add (BUFFER* buf, const char* s, size_t len)
 
 void mutt_buffer_addstr (BUFFER* buf, const char* s)
 {
-  mutt_buffer_add (buf, s, mutt_strlen (s));
+  mutt_buffer_addstr_n (buf, s, mutt_strlen (s));
 }
 
 void mutt_buffer_addch (BUFFER* buf, char c)
 {
-  mutt_buffer_add (buf, &c, 1);
+  mutt_buffer_addstr_n (buf, &c, 1);
 }
 
 void mutt_buffer_strcpy (BUFFER *buf, const char *s)
@@ -197,6 +202,12 @@ void mutt_buffer_strcpy (BUFFER *buf, const char *s)
   mutt_buffer_addstr (buf, s);
 }
 
+void mutt_buffer_strcpy_n (BUFFER *buf, const char *s, size_t len)
+{
+  mutt_buffer_clear (buf);
+  mutt_buffer_addstr_n (buf, s, len);
+}
+
 
 static void increase_buffer_pool (void)
 {
