@@ -1,20 +1,20 @@
 /*
  * Copyright (C) 1996-2000,2007 Michael R. Elkins <me@mutt.org>
- * 
+ *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
  *     the Free Software Foundation; either version 2 of the License, or
  *     (at your option) any later version.
- * 
+ *
  *     This program is distributed in the hope that it will be useful,
  *     but WITHOUT ANY WARRANTY; without even the implied warranty of
  *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *     GNU General Public License for more details.
- * 
+ *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */ 
+ */
 
 #if HAVE_CONFIG_H
 # include "config.h"
@@ -67,7 +67,7 @@ int mutt_complete (char *s, size_t slen)
   if (mx_is_imap (imap_path))
     return imap_complete (s, slen, imap_path);
 #endif
-  
+
   if (*s == '=' || *s == '+' || *s == '!')
   {
     dirpart[0] = *s;
@@ -79,8 +79,10 @@ int mutt_complete (char *s, size_t slen)
     if ((p = strrchr (s, '/')))
     {
       char buf[_POSIX_PATH_MAX];
-      if (mutt_concatn_path (buf, sizeof(buf), exp_dirpart, strlen(exp_dirpart), s + 1, (size_t)(p - s - 1)) == NULL) {
-	      return -1;
+      if (mutt_concatn_path (buf, sizeof(buf), exp_dirpart, strlen(exp_dirpart),
+                             s + 1, (size_t)(p - s - 1)) == NULL)
+      {
+        return -1;
       }
       strfcpy (exp_dirpart, buf, sizeof (exp_dirpart));
       mutt_substrcpy(dirpart, s, p+1, sizeof(dirpart));
