@@ -1,20 +1,20 @@
 /*
  * Copyright (C) 1996-2000,2002 Michael R. Elkins <me@mutt.org>
- * 
+ *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
  *     the Free Software Foundation; either version 2 of the License, or
  *     (at your option) any later version.
- * 
+ *
  *     This program is distributed in the hope that it will be useful,
  *     but WITHOUT ANY WARRANTY; without even the implied warranty of
  *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *     GNU General Public License for more details.
- * 
+ *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */ 
+ */
 
 /*
  * This file contains the structures needed to parse ``bind'' commands, as
@@ -106,6 +106,7 @@ const struct binding_t OpMain[] = { /* map: index */
   { "edit-type",		OP_EDIT_TYPE,			"\005" },
   { "forward-message",		OP_FORWARD_MESSAGE,		"f" },
   { "flag-message",		OP_FLAG_MESSAGE,		"F" },
+  { "group-chat-reply",		OP_GROUP_CHAT_REPLY,		NULL },
   { "group-reply",		OP_GROUP_REPLY,			"g" },
 #ifdef USE_POP
   { "fetch-mail",		OP_MAIN_FETCH_MAIL,		"G" },
@@ -209,6 +210,7 @@ const struct binding_t OpPager[] = { /* map: pager */
   { "edit-type",	OP_EDIT_TYPE,			"\005" },
   { "forward-message",	OP_FORWARD_MESSAGE,		"f" },
   { "flag-message",	OP_FLAG_MESSAGE,		"F" },
+  { "group-chat-reply",	OP_GROUP_CHAT_REPLY,		NULL },
   { "group-reply",	OP_GROUP_REPLY,			"g" },
 #ifdef USE_IMAP
   { "imap-fetch-mail",  OP_MAIN_IMAP_FETCH,		NULL },
@@ -224,6 +226,7 @@ const struct binding_t OpPager[] = { /* map: pager */
   { "redraw-screen",	OP_REDRAW,			"\014" },
   { "mail",		OP_MAIL,			"m" },
   { "mark-as-new",	OP_TOGGLE_NEW,			"N" },
+  { "toggle-write",	OP_TOGGLE_WRITE,		"%" },
   { "search-next",	OP_SEARCH_NEXT,			"n" },
   { "next-thread",	OP_MAIN_NEXT_THREAD,		"\016" },
   { "next-subthread",	OP_MAIN_NEXT_SUBTHREAD,		"\033n" },
@@ -253,7 +256,7 @@ const struct binding_t OpPager[] = { /* map: pager */
   { "search-toggle",	OP_SEARCH_TOGGLE,		"\\" },
   { "display-address",	OP_DISPLAY_ADDRESS,		"@" },
   { "next-new",		OP_MAIN_NEXT_NEW,		NULL },
-  { "next-new-then-unread", 
+  { "next-new-then-unread",
                         OP_MAIN_NEXT_NEW_THEN_UNREAD,   "\t" },
   { "pipe-message",	OP_PIPE,			"|" },
   { "help",		OP_HELP,			"?" },
@@ -320,6 +323,7 @@ const struct binding_t OpAttach[] = { /* map: attachment */
   { "view-mailcap",	OP_ATTACH_VIEW_MAILCAP,		"m" },
   { "reply",		OP_REPLY,			"r" },
   { "resend-message",	OP_RESEND,			"\033e" },
+  { "group-chat-reply",	OP_GROUP_CHAT_REPLY,		NULL },
   { "group-reply",	OP_GROUP_REPLY,			"g" },
   { "list-reply",	OP_LIST_REPLY,			"L" },
   { "forward-message",	OP_FORWARD_MESSAGE,		"f" },
@@ -383,7 +387,7 @@ const struct binding_t OpCompose[] = { /* map: compose */
 #ifdef MIXMASTER
   { "mix",		OP_COMPOSE_MIX,			"M" },
 #endif
-  
+
   { NULL,		0,				NULL }
 };
 
@@ -398,11 +402,12 @@ const struct binding_t OpAlias[] = { /* map: alias */
   { "undelete-entry",	OP_UNDELETE,	"u" },
   { NULL,		0,		NULL }
 };
-  
+
 
 /* The file browser */
 const struct binding_t OpBrowser[] = { /* map: browser */
   { "change-dir",	OP_CHANGE_DIRECTORY,	"c" },
+  { "descend-directory", OP_DESCEND_DIRECTORY,	NULL },
   { "display-filename",	OP_BROWSER_TELL,	"@" },
   { "enter-mask",	OP_ENTER_MASK,		"m" },
   { "sort",		OP_SORT,		"o" },
