@@ -27,6 +27,7 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <ctype.h>
+#include <limits.h>
 
 #include "mutt.h"
 #include "imap_private.h"
@@ -68,7 +69,7 @@ static int msg_cache_commit (IMAP_DATA* idata, HEADER* h);
 
 static void flush_buffer(char* buf, size_t* len, CONNECTION* conn);
 static int msg_fetch_header (CONTEXT* ctx, IMAP_HEADER* h, char* buf,
-  FILE* fp);
+                             FILE* fp);
 static int msg_parse_fetch (IMAP_HEADER* h, char* s);
 static char* msg_parse_flags (IMAP_HEADER* h, char* s);
 
@@ -137,22 +138,41 @@ static void imap_alloc_uid_hash (IMAP_DATA *idata, unsigned int msn_count)
 
 /* Generates a more complicated sequence set after using the header cache,
  * in case there are missing MSNs in the middle.
- *
- * There is a suggested limit of 1000 bytes for an IMAP client request.
- * Ideally, we would generate multiple requests if the number of ranges
- * is too big, but for now just abort to using the whole range.
  */
-static void imap_fetch_msn_seqset (BUFFER *b, IMAP_DATA *idata, unsigned int msn_begin,
-                                   unsigned int msn_end)
+static unsigned int imap_fetch_msn_seqset (BUFFER *b, IMAP_DATA *idata, int evalhc,
+                                           unsigned int msn_begin, unsigned int msn_end,
+                                           unsigned int *fetch_msn_end)
 {
-  int chunks = 0;
+  unsigned int max_headers_per_fetch = UINT_MAX;
+  int first_chunk = 1;
   int state = 0;  /* 1: single msn, 2: range of msn */
-  unsigned int msn, range_begin, range_end;
+  unsigned int msn, range_begin, range_end, msn_count = 0;
+
+  mutt_buffer_clear (b);
+  if (msn_end < msn_begin)
+    return 0;
+
+  if (ImapFetchChunkSize > 0)
+    max_headers_per_fetch = ImapFetchChunkSize;
+
+  if (!evalhc)
+  {
+    if (msn_end - msn_begin + 1 <= max_headers_per_fetch)
+      *fetch_msn_end = msn_end;
+    else
+      *fetch_msn_end = msn_begin + max_headers_per_fetch - 1;
+    mutt_buffer_printf (b, "%u:%u", msn_begin, *fetch_msn_end);
+    return (*fetch_msn_end - msn_begin + 1);
+  }
 
   for (msn = msn_begin; msn <= msn_end + 1; msn++)
   {
-    if (msn <= msn_end && !idata->msn_index[msn-1])
+    if (msn_count < max_headers_per_fetch &&
+        msn <= msn_end &&
+        !idata->msn_index[msn-1])
     {
+      msn_count++;
+
       switch (state)
       {
         case 1:            /* single: convert to a range */
@@ -169,25 +189,27 @@ static void imap_fetch_msn_seqset (BUFFER *b, IMAP_DATA *idata, unsigned int msn
     }
     else if (state)
     {
-      if (chunks++)
+      if (first_chunk)
+        first_chunk = 0;
+      else
         mutt_buffer_addch (b, ',');
-      if (chunks == 150)
-        break;
 
       if (state == 1)
         mutt_buffer_add_printf (b, "%u", range_begin);
       else if (state == 2)
         mutt_buffer_add_printf (b, "%u:%u", range_begin, range_end);
       state = 0;
+
+      if ((mutt_buffer_len (b) > 500) ||
+          msn_count >= max_headers_per_fetch)
+        break;
     }
   }
 
-  /* Too big.  Just query the whole range then. */
-  if (chunks == 150 || mutt_strlen (b->data) > 500)
-  {
-    b->dptr = b->data;
-    mutt_buffer_add_printf (b, "%u:%u", msn_begin, msn_end);
-  }
+  /* The loop index goes one past to terminate the range if needed. */
+  *fetch_msn_end = msn - 1;
+
+  return msn_count;
 }
 
 /* imap_read_headers:
@@ -686,7 +708,7 @@ static int read_headers_fetch_new (IMAP_DATA *idata, unsigned int msn_begin,
   char tempfile[_POSIX_PATH_MAX];
   FILE *fp = NULL;
   IMAP_HEADER h;
-  BUFFER *b;
+  BUFFER *b = NULL;
   static const char * const want_headers = "DATE FROM SENDER SUBJECT TO CC MESSAGE-ID REFERENCES CONTENT-TYPE CONTENT-DESCRIPTION IN-REPLY-TO REPLY-TO LINES LIST-POST X-LABEL";
 
   ctx = idata->ctx;
@@ -695,12 +717,12 @@ static int read_headers_fetch_new (IMAP_DATA *idata, unsigned int msn_begin,
   if (mutt_bit_isset (idata->capabilities,IMAP4REV1))
   {
     safe_asprintf (&hdrreq, "BODY.PEEK[HEADER.FIELDS (%s%s%s)]",
-                           want_headers, ImapHeaders ? " " : "", NONULL (ImapHeaders));
+                   want_headers, ImapHeaders ? " " : "", NONULL (ImapHeaders));
   }
   else if (mutt_bit_isset (idata->capabilities,IMAP4))
   {
     safe_asprintf (&hdrreq, "RFC822.HEADER.LINES (%s%s%s)",
-                           want_headers, ImapHeaders ? " " : "", NONULL (ImapHeaders));
+                   want_headers, ImapHeaders ? " " : "", NONULL (ImapHeaders));
   }
   else
   {	/* Unable to fetch headers for lower versions */
@@ -723,24 +745,26 @@ static int read_headers_fetch_new (IMAP_DATA *idata, unsigned int msn_begin,
   mutt_progress_init (&progress, _("Fetching message headers..."),
 		      MUTT_PROGRESS_MSG, ReadInc, msn_end);
 
-  while (msn_begin <= msn_end && fetch_msn_end < msn_end)
-  {
-    b = mutt_buffer_new ();
-    if (evalhc)
-    {
-      /* In case there are holes in the header cache. */
-      evalhc = 0;
-      imap_fetch_msn_seqset (b, idata, msn_begin, msn_end);
-    }
-    else
-      mutt_buffer_add_printf (b, "%u:%u", msn_begin, msn_end);
+  b = mutt_buffer_pool_get ();
 
-    fetch_msn_end = msn_end;
+  /* NOTE:
+   *   The (fetch_msn_end < msn_end) used to be important to prevent
+   *   an infinite loop, in the event the server did not return all
+   *   the headers (due to a pending expunge, for example).
+   *
+   *   I believe the new chunking imap_fetch_msn_seqset()
+   *   implementation and "msn_begin = fetch_msn_end + 1" assignment
+   *   at the end of the loop makes the comparison unneeded, but to be
+   *   cautious I'm keeping it.
+   */
+  while ((fetch_msn_end < msn_end) &&
+         imap_fetch_msn_seqset (b, idata, evalhc, msn_begin, msn_end,
+                                &fetch_msn_end))
+  {
     safe_asprintf (&cmd, "FETCH %s (UID FLAGS INTERNALDATE RFC822.SIZE %s)",
-                   b->data, hdrreq);
+                   mutt_b2s (b), hdrreq);
     imap_cmd_start (idata, cmd);
     FREE (&cmd);
-    mutt_buffer_free (&b);
 
     rc = IMAP_CMD_CONTINUE;
     for (msgno = msn_begin; rc == IMAP_CMD_CONTINUE; msgno++)
@@ -840,17 +864,9 @@ static int read_headers_fetch_new (IMAP_DATA *idata, unsigned int msn_begin,
         goto bail;
     }
 
-    /* In case we get new mail while fetching the headers.
-     *
-     * Note: The RFC says we shouldn't get any EXPUNGE responses in the
-     * middle of a FETCH.  But just to be cautious, use the current state
-     * of max_msn, not fetch_msn_end to set the next start range.
-     */
+    /* In case we get new mail while fetching the headers. */
     if (idata->reopen & IMAP_NEWMAIL_PENDING)
     {
-      /* update to the last value we actually pulled down */
-      fetch_msn_end = idata->max_msn;
-      msn_begin = idata->max_msn + 1;
       msn_end = idata->newMailCount;
       while (msn_end > ctx->hdrmax)
         mx_alloc_memory (ctx);
@@ -858,11 +874,24 @@ static int read_headers_fetch_new (IMAP_DATA *idata, unsigned int msn_begin,
       idata->reopen &= ~IMAP_NEWMAIL_PENDING;
       idata->newMailCount = 0;
     }
+
+    /* Note: RFC3501 section 7.4.1 and RFC7162 section 3.2.10.2 say we
+     * must not get any EXPUNGE/VANISHED responses in the middle of a
+     * FETCH, nor when no command is in progress (e.g. between the
+     * chunked FETCH commands).  We previously tried to be robust by
+     * setting:
+     *   msn_begin = idata->max_msn + 1;
+     * but with chunking (and the mythical header cache holes) this
+     * may not be correct.  So here we must assume the msn values have
+     * not been altered during or after the fetch.
+     */
+    msn_begin = fetch_msn_end + 1;
   }
 
   retval = 0;
 
 bail:
+  mutt_buffer_pool_release (&b);
   safe_fclose (&fp);
   FREE (&hdrreq);
 
@@ -1027,7 +1056,7 @@ int imap_fetch_message (CONTEXT *ctx, MESSAGE *msg, int msgno)
 
   msg_cache_commit (idata, h);
 
-  parsemsg:
+parsemsg:
   /* Update the header information.  Previously, we only downloaded a
    * portion of the headers, those required for the main display.
    */
@@ -1071,6 +1100,7 @@ int imap_fetch_message (CONTEXT *ctx, MESSAGE *msg, int msgno)
   return 0;
 
 bail:
+  h->active = 1;
   safe_fclose (&msg->fp);
   imap_cache_del (idata, h);
   if (cache->path)
@@ -1136,7 +1166,7 @@ int imap_append_message (CONTEXT *ctx, MESSAGE *msg)
    * Ideally we'd have a HEADER structure with flag info here... */
   for (last = EOF, len = 0; (c = fgetc(fp)) != EOF; last = c)
   {
-    if(c == '\n' && last != '\r')
+    if (c == '\n' && last != '\r')
       len++;
 
     len++;
@@ -1228,7 +1258,7 @@ int imap_append_message (CONTEXT *ctx, MESSAGE *msg)
   FREE (&mx.mbox);
   return 0;
 
- fail:
+fail:
   FREE (&mx.mbox);
   return -1;
 }
@@ -1264,7 +1294,7 @@ int imap_copy_messages (CONTEXT* ctx, HEADER* h, char* dest, int delete)
   if (!mutt_account_match (&(CTX_DATA->conn->account), &(mx.account)))
   {
     dprint (3, (debugfile, "imap_copy_messages: %s not same server as %s\n",
-      dest, ctx->path));
+                dest, ctx->path));
     return 1;
   }
 
@@ -1404,7 +1434,7 @@ int imap_copy_messages (CONTEXT* ctx, HEADER* h, char* dest, int delete)
 
   rc = 0;
 
- out:
+out:
   if (cmd.data)
     FREE (&cmd.data);
   if (sync_cmd.data)
@@ -1755,7 +1785,7 @@ static int msg_parse_fetch (IMAP_HEADER *h, char *s)
         return -1;
     }
     else if (!ascii_strncasecmp ("BODY", s, 4) ||
-      !ascii_strncasecmp ("RFC822.HEADER", s, 13))
+             !ascii_strncasecmp ("RFC822.HEADER", s, 13))
     {
       /* handle above, in msg_fetch_header */
       return -2;
@@ -1804,7 +1834,7 @@ static char* msg_parse_flags (IMAP_HEADER* h, char* s)
   if (ascii_strncasecmp ("FLAGS", s, 5) != 0)
   {
     dprint (1, (debugfile, "msg_parse_flags: not a FLAGS response: %s\n",
-      s));
+                s));
     return NULL;
   }
   s += 5;
@@ -1812,7 +1842,7 @@ static char* msg_parse_flags (IMAP_HEADER* h, char* s)
   if (*s != '(')
   {
     dprint (1, (debugfile, "msg_parse_flags: bogus FLAGS response: %s\n",
-      s));
+                s));
     return NULL;
   }
   s++;
@@ -1875,7 +1905,7 @@ static char* msg_parse_flags (IMAP_HEADER* h, char* s)
   else
   {
     dprint (1, (debugfile,
-      "msg_parse_flags: Unterminated FLAGS response: %s\n", s));
+                "msg_parse_flags: Unterminated FLAGS response: %s\n", s));
     return NULL;
   }
 
