@@ -2,17 +2,17 @@
  * Copyright (C) 1996-2002,2007,2009 Michael R. Elkins <me@mutt.org>
  * Copyright (C) 1999-2005 Thomas Roessler <roessler@does-not-exist.org>
  * Copyright (C) 2010,2013 Michael R. Elkins <me@mutt.org>
- * 
+ *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
  *     the Free Software Foundation; either version 2 of the License, or
  *     (at your option) any later version.
- * 
+ *
  *     This program is distributed in the hope that it will be useful,
  *     but WITHOUT ANY WARRANTY; without even the implied warranty of
  *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *     GNU General Public License for more details.
- * 
+ *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
@@ -248,7 +248,7 @@ static int mh_sequences_changed(BUFFY *b)
   int rc = -1;
 
   path = mutt_buffer_pool_get ();
-  mutt_buffer_printf (path, "%s/.mh_sequences", b->path);
+  mutt_buffer_printf (path, "%s/.mh_sequences", mutt_b2s (b->pathbuf));
   if (stat (mutt_b2s (path), &sb) == 0)
     rc = (mutt_stat_timespec_compare (&sb, MUTT_STAT_MTIME, &b->last_visited) > 0);
   mutt_buffer_pool_release (&path);
@@ -267,7 +267,7 @@ static int mh_already_notified(BUFFY *b, int msgno)
   int rc = -1;
 
   path = mutt_buffer_pool_get ();
-  mutt_buffer_printf (path, "%s/%d", b->path, msgno);
+  mutt_buffer_printf (path, "%s/%d", mutt_b2s (b->pathbuf), msgno);
   if (stat (mutt_b2s (path), &sb) == 0)
     rc = (mutt_stat_timespec_compare (&sb, MUTT_STAT_MTIME, &b->last_visited) <= 0);
   mutt_buffer_pool_release (&path);
@@ -299,7 +299,7 @@ int mh_buffy (BUFFY *mailbox, int check_stats)
     return rc;
 
   memset (&mhs, 0, sizeof (mhs));
-  if (mh_read_sequences (&mhs, mailbox->path) < 0)
+  if (mh_read_sequences (&mhs, mutt_b2s (mailbox->pathbuf)) < 0)
     return 0;
 
   if (check_stats)
@@ -340,7 +340,7 @@ int mh_buffy (BUFFY *mailbox, int check_stats)
 
   if (check_stats)
   {
-    if ((dirp = opendir (mailbox->path)) != NULL)
+    if ((dirp = opendir (mutt_b2s (mailbox->pathbuf))) != NULL)
     {
       while ((de = readdir (dirp)) != NULL)
       {
@@ -368,7 +368,7 @@ static int mh_mkstemp (CONTEXT * dest, FILE ** fp, char **tgt)
   FOREVER
   {
     mutt_buffer_printf (path, "%s/.mutt-%s-%d-%d",
-	      dest->path, NONULL (Hostname), (int) getpid (), Counter++);
+                        dest->path, NONULL (Hostname), (int) getpid (), Counter++);
     if ((fd = open (mutt_b2s (path), O_WRONLY | O_EXCL | O_CREAT, 0666)) == -1)
     {
       if (errno != EEXIST)
@@ -693,7 +693,7 @@ static void maildir_parse_flags (HEADER * h, const char *path)
   if ((p = strrchr (path, ':')) != NULL && mutt_strncmp (p + 1, "2,", 2) == 0)
   {
     p += 3;
-    
+
     mutt_str_replace (&h->maildir_flags, p);
     q = h->maildir_flags;
 
@@ -701,37 +701,37 @@ static void maildir_parse_flags (HEADER * h, const char *path)
     {
       switch (*p)
       {
-      case 'F':
+        case 'F':
 
-	h->flagged = 1;
-	break;
+          h->flagged = 1;
+          break;
 
-      case 'S':		/* seen */
+        case 'S':		/* seen */
 
-	h->read = 1;
-	break;
+          h->read = 1;
+          break;
 
-      case 'R':		/* replied */
+        case 'R':		/* replied */
 
-	h->replied = 1;
-	break;
+          h->replied = 1;
+          break;
 
-      case 'T':		/* trashed */
-	if (!h->flagged || !option(OPTFLAGSAFE))
-	{
-	  h->trash = 1;
-	  h->deleted = 1;
-	}
-	break;
-      
-      default:
-	*q++ = *p;
-	break;
+        case 'T':		/* trashed */
+          if (!h->flagged || !option(OPTFLAGSAFE))
+          {
+            h->trash = 1;
+            h->deleted = 1;
+          }
+          break;
+
+        default:
+          *q++ = *p;
+          break;
       }
       p++;
     }
   }
-  
+
   if (q == h->maildir_flags)
     FREE (&h->maildir_flags);
   else if (q)
@@ -768,7 +768,7 @@ static void maildir_update_mtime (CONTEXT * ctx)
   mutt_buffer_pool_release (&buf);
 }
 
-/* 
+/*
  * Actually parse a maildir message.  This may also be used to fill
  * out a fake header structure generated by lazy maildir parsing.
  */
@@ -798,9 +798,9 @@ static HEADER *maildir_parse_message (int magic, const char *fname,
 
     if (magic == MUTT_MAILDIR)
     {
-      /* 
+      /*
        * maildir stores its flags in the filename, so ignore the
-       * flags in the header of the message 
+       * flags in the header of the message
        */
 
       h->old = is_old;
@@ -981,37 +981,37 @@ static struct maildir*  maildir_merge_lists (struct maildir *left,
   struct maildir* head;
   struct maildir* tail;
 
-  if (left && right) 
+  if (left && right)
   {
     if (cmp (left, right) < 0)
     {
       head = left;
       left = left->next;
     }
-    else 
+    else
     {
       head = right;
       right = right->next;
     }
-  } 
-  else 
+  }
+  else
   {
-    if (left) 
+    if (left)
       return left;
-    else 
+    else
       return right;
   }
-    
+
   tail = head;
 
-  while (left && right) 
+  while (left && right)
   {
     if (cmp (left, right) < 0)
     {
       tail->next = left;
       left = left->next;
-    } 
-    else 
+    }
+    else
     {
       tail->next = right;
       right = right->next;
@@ -1019,7 +1019,7 @@ static struct maildir*  maildir_merge_lists (struct maildir *left,
     tail = tail->next;
   }
 
-  if (left) 
+  if (left)
   {
     tail->next = left;
   }
@@ -1071,7 +1071,7 @@ static struct maildir* maildir_sort (struct maildir* list, size_t len,
   struct maildir* right = list;
   size_t c = 0;
 
-  if (!list || !list->next) 
+  if (!list || !list->next)
   {
     return list;
   }
@@ -1080,7 +1080,7 @@ static struct maildir* maildir_sort (struct maildir* list, size_t len,
     return maildir_ins_sort (list, cmp);
 
   list = list->next;
-  while (list && list->next) 
+  while (list && list->next)
   {
     right = right->next;
     list = list->next->next;
@@ -1121,7 +1121,8 @@ static struct maildir *skip_duplicates (struct maildir *p, struct maildir **last
    * likely be at the head of the list.  but it is present for consistency with
    * the check at the top of the for() loop in maildir_delayed_parsing().
    */
-  while (!p->h || p->header_parsed) {
+  while (!p->h || p->header_parsed)
+  {
     *last = p;
     p = p->next;
   }
@@ -1129,12 +1130,12 @@ static struct maildir *skip_duplicates (struct maildir *p, struct maildir **last
 }
 #endif
 
-/* 
+/*
  * This function does the second parsing pass
  */
 static void maildir_delayed_parsing (CONTEXT * ctx, struct maildir **md,
-			      progress_t *progress)
-{ 
+                                     progress_t *progress)
+{
   struct maildir *p, *last = NULL;
   BUFFER *fn = NULL;
   int count;
@@ -1150,20 +1151,22 @@ static void maildir_delayed_parsing (CONTEXT * ctx, struct maildir **md,
 #endif
 
 #if HAVE_DIRENT_D_INO
-#define DO_SORT()	do { \
-  if (!sort) \
-  { \
-    dprint (4, (debugfile, "maildir: need to sort %s by inode\n", ctx->path)); \
-    p = maildir_sort (p, (size_t) -1, md_cmp_inode); \
-    if (!last) \
-      *md = p; \
-    else \
-      last->next = p; \
-    sort = 1; \
-    p = skip_duplicates (p, &last); \
-    mutt_buffer_printf (fn, "%s/%s", ctx->path, p->h->path); \
-  } \
-} while(0)
+#define DO_SORT()                                                       \
+  do                                                                    \
+  {                                                                     \
+    if (!sort)                                                          \
+    {                                                                   \
+      dprint (4, (debugfile, "maildir: need to sort %s by inode\n", ctx->path)); \
+      p = maildir_sort (p, (size_t) -1, md_cmp_inode);                  \
+      if (!last)                                                        \
+        *md = p;                                                        \
+      else                                                              \
+        last->next = p;                                                 \
+      sort = 1;                                                         \
+      p = skip_duplicates (p, &last);                                   \
+      mutt_buffer_printf (fn, "%s/%s", ctx->path, p->h->path);          \
+    }                                                                   \
+  } while (0)
 #else
 #define DO_SORT()	/* nothing */
 #endif
@@ -1175,9 +1178,9 @@ static void maildir_delayed_parsing (CONTEXT * ctx, struct maildir **md,
   fn = mutt_buffer_pool_get ();
 
   for (p = *md, count = 0; p; p = p->next, count++)
-   {
+  {
     if (! (p && p->h && !p->header_parsed))
-     {
+    {
       last = p;
       continue;
     }
@@ -1216,23 +1219,24 @@ static void maildir_delayed_parsing (CONTEXT * ctx, struct maildir **md,
     {
 #endif /* USE_HCACHE */
 
-    if (maildir_parse_message (ctx->magic, mutt_b2s (fn), p->h->old, p->h))
-    {
-      p->header_parsed = 1;
+      if (maildir_parse_message (ctx->magic, mutt_b2s (fn), p->h->old, p->h))
+      {
+        p->header_parsed = 1;
 #if USE_HCACHE
-      if (ctx->magic == MUTT_MH)
-	mutt_hcache_store (hc, p->h->path, p->h, 0, strlen, MUTT_GENERATE_UIDVALIDITY);
-      else
-	mutt_hcache_store (hc, p->h->path + 3, p->h, 0, &maildir_hcache_keylen, MUTT_GENERATE_UIDVALIDITY);
+        if (ctx->magic == MUTT_MH)
+          mutt_hcache_store (hc, p->h->path, p->h, 0, strlen, MUTT_GENERATE_UIDVALIDITY);
+        else
+          mutt_hcache_store (hc, p->h->path + 3, p->h, 0, &maildir_hcache_keylen, MUTT_GENERATE_UIDVALIDITY);
 #endif
-    } else
-      mutt_free_header (&p->h);
+      }
+      else
+        mutt_free_header (&p->h);
 #if USE_HCACHE
     }
     mutt_hcache_free (&data);
 #endif
     last = p;
-   }
+  }
 #if USE_HCACHE
   mutt_hcache_close (hc);
 #endif
@@ -1446,7 +1450,7 @@ static void maildir_flags (char *dest, size_t destlen, HEADER * hdr)
    * will end up in the cur directory, so we include it in the following
    * test even though there is no associated flag.
    */
-  
+
   if (hdr && (hdr->flagged || hdr->replied || hdr->read || hdr->deleted || hdr->old || hdr->maildir_flags))
   {
     char tmp[LONG_STRING];
@@ -1506,7 +1510,7 @@ static int mh_close_message (CONTEXT *ctx, MESSAGE *msg)
 
 /*
  * Open a new (temporary) message in a maildir folder.
- * 
+ *
  * Note that this uses _almost_ the maildir file name format, but
  * with a {cur,new} prefix.
  *
@@ -1543,8 +1547,8 @@ static int maildir_open_new_message (MESSAGE * msg, CONTEXT * dest, HEADER * hdr
   FOREVER
   {
     mutt_buffer_printf (path, "%s/tmp/%s.%lld.%u_%d.%s%s",
-	      dest->path, subdir, (long long)time (NULL), (unsigned int)getpid (),
-	      Counter++, NONULL (Hostname), suffix);
+                        dest->path, subdir, (long long)time (NULL), (unsigned int)getpid (),
+                        Counter++, NONULL (Hostname), suffix);
 
     dprint (2, (debugfile, "maildir_open_new_message (): Trying %s.\n",
 		mutt_b2s (path)));
@@ -1587,23 +1591,23 @@ out:
 
 /*
  * Commit a message to a maildir folder.
- * 
+ *
  * msg->path contains the file name of a file in tmp/. We take the
- * flags from this file's name. 
+ * flags from this file's name.
  *
  * ctx is the mail folder we commit to.
- * 
+ *
  * hdr is a header structure to which we write the message's new
  * file name.  This is used in the mh and maildir folder synch
  * routines.  When this routine is invoked from mx_commit_message,
- * hdr is NULL. 
+ * hdr is NULL.
  *
  * msg->path looks like this:
- * 
+ *
  *    tmp/{cur,new}.mutt-HOSTNAME-PID-COUNTER:flags
- * 
+ *
  * See also maildir_open_new_message().
- * 
+ *
  */
 
 static int _maildir_commit_message (CONTEXT * ctx, MESSAGE * msg, HEADER * hdr)
@@ -1636,8 +1640,8 @@ static int _maildir_commit_message (CONTEXT * ctx, MESSAGE * msg, HEADER * hdr)
   FOREVER
   {
     mutt_buffer_printf (path, "%s/%lld.%u_%d.%s%s", subdir,
-	      (long long)time (NULL), (unsigned int)getpid (), Counter++,
-	      NONULL (Hostname), suffix);
+                        (long long)time (NULL), (unsigned int)getpid (), Counter++,
+                        NONULL (Hostname), suffix);
     mutt_buffer_printf (full, "%s/%s", ctx->path, mutt_b2s (path));
 
     dprint (2, (debugfile, "_maildir_commit_message (): renaming %s to %s.\n",
@@ -1690,9 +1694,9 @@ static int maildir_commit_message (CONTEXT * ctx, MESSAGE * msg)
   return _maildir_commit_message (ctx, msg, NULL);
 }
 
-/* 
+/*
  * commit a message to an MH folder.
- * 
+ *
  */
 
 
@@ -1741,9 +1745,9 @@ static int _mh_commit_message (CONTEXT * ctx, MESSAGE * msg, HEADER * hdr,
   }
   closedir (dirp);
 
-  /* 
+  /*
    * Now try to rename the file to the proper name.
-   * 
+   *
    * Note: We may have to try multiple times, until we find a free
    * slot.
    */
@@ -1786,7 +1790,7 @@ static int mh_commit_message (CONTEXT * ctx, MESSAGE * msg)
 
 
 /* Sync a message in an MH folder.
- * 
+ *
  * This code is also used for attachment deletion in maildir
  * folders.
  */
@@ -1831,7 +1835,7 @@ static int mh_rewrite_message (CONTEXT * ctx, int msgno)
       restore = 0;
     }
 
-    /* 
+    /*
      * Try to move the new message to the old place.
      * (MH only.)
      *
@@ -1840,7 +1844,7 @@ static int mh_rewrite_message (CONTEXT * ctx, int msgno)
      * Note that there is a race condition against programs which
      * use the first free slot instead of the maximum message
      * number.  Mutt does _not_ behave like this.
-     * 
+     *
      * Anyway, if this fails, the message is in the folder, so
      * all what happens is that a concurrently running mutt will
      * lose flag modifications.
@@ -1878,10 +1882,15 @@ static int mh_sync_message (CONTEXT * ctx, int msgno)
 {
   HEADER *h = ctx->hdrs[msgno];
 
-  if (h->attach_del || h->xlabel_changed ||
-      (h->env && (h->env->refs_changed || h->env->irt_changed)))
+  /* TODO: why the h->env check? */
+  if (h->attach_del || (h->env && h->env->changed))
+  {
     if (mh_rewrite_message (ctx, msgno) != 0)
       return -1;
+    /* TODO: why the env check? */
+    if (h->env)
+      h->env->changed = 0;
+  }
 
   return 0;
 }
@@ -1897,12 +1906,15 @@ static int maildir_sync_message (CONTEXT * ctx, int msgno)
   char *p;
   int rc = 0;
 
-  if (h->attach_del || h->xlabel_changed ||
-      (h->env && (h->env->refs_changed || h->env->irt_changed)))
+  /* TODO: why the h->env check? */
+  if (h->attach_del || (h->env && h->env->changed))
   {
     /* when doing attachment deletion/rethreading, fall back to the MH case. */
     if (mh_rewrite_message (ctx, msgno) != 0)
       return (-1);
+    /* TODO: why the env check? */
+    if (h->env)
+      h->env->changed = 0;
   }
   else
   {
@@ -1979,9 +1991,9 @@ int mh_sync_mailbox (CONTEXT * ctx, int *index_hint)
 
   if (ctx->magic == MUTT_MH)
     i = mh_check_mailbox (ctx, index_hint);
-  else 
+  else
     i = maildir_check_mailbox (ctx, index_hint);
-      
+
   if (i != 0)
     return i;
 
@@ -2025,7 +2037,7 @@ int mh_sync_mailbox (CONTEXT * ctx, int *index_hint)
 	if (*ctx->hdrs[i]->path != ',')
 	{
 	  mutt_buffer_printf (tmp, "%s/,%s", ctx->path,
-		    ctx->hdrs[i]->path);
+                              ctx->hdrs[i]->path);
 	  unlink (mutt_b2s (tmp));
 	  rename (mutt_b2s (path), mutt_b2s (tmp));
 	}
@@ -2033,7 +2045,6 @@ int mh_sync_mailbox (CONTEXT * ctx, int *index_hint)
       }
     }
     else if (ctx->hdrs[i]->changed || ctx->hdrs[i]->attach_del ||
-	     ctx->hdrs[i]->xlabel_changed ||
 	     (ctx->magic == MUTT_MAILDIR
 	      && (option (OPTMAILDIRTRASH) || ctx->hdrs[i]->trash)
 	      && (ctx->hdrs[i]->deleted != ctx->hdrs[i]->trash)))
@@ -2122,7 +2133,7 @@ static void maildir_update_tables (CONTEXT *ctx, int *index_hint)
   short old_sort;
   int old_count;
   int i, j;
-  
+
   if (Sort != SORT_ORDER)
   {
     old_sort = Sort;
@@ -2130,13 +2141,13 @@ static void maildir_update_tables (CONTEXT *ctx, int *index_hint)
     mutt_sort_headers (ctx, 1);
     Sort = old_sort;
   }
-  
+
   old_count = ctx->msgcount;
   for (i = 0, j = 0; i < old_count; i++)
   {
     if (ctx->hdrs[i]->active && index_hint && *index_hint == i)
       *index_hint = j;
-    
+
     if (ctx->hdrs[i]->active)
       ctx->hdrs[i]->index = j++;
   }
@@ -2152,7 +2163,7 @@ static int maildir_update_flags (CONTEXT *ctx, HEADER *o, HEADER *n)
    */
   int context_changed = ctx->changed;
   int header_changed;
-  
+
   /* user didn't modify this message.  alter the flags to match the
    * current state on disk.  This may not actually do
    * anything. mutt_set_flag() will just ignore the call if the status
@@ -2174,7 +2185,7 @@ static int maildir_update_flags (CONTEXT *ctx, HEADER *o, HEADER *n)
    */
   header_changed = o->changed;
   o->changed = 0;
-  
+
   /* if the mailbox was not modified before we made these
    * changes, unset the changed flag since nothing needs to
    * be synchronized.
@@ -2348,7 +2359,7 @@ static int maildir_check_mailbox (CONTEXT * ctx, int *index_hint)
   /* If we didn't just get new mail, update the tables. */
   if (occult)
     maildir_update_tables (ctx, index_hint);
-  
+
   /* do any delayed parsing we need to do. */
   maildir_delayed_parsing (ctx, &md, NULL);
 
@@ -2366,7 +2377,7 @@ static int maildir_check_mailbox (CONTEXT * ctx, int *index_hint)
   return 0;
 }
 
-/* 
+/*
  * This function handles arrival of new mail and reopening of
  * mh/maildir folders. Things are getting rather complex because we
  * don't have a well-defined "mailbox order", so the tricks from
@@ -2400,14 +2411,14 @@ static int mh_check_mailbox (CONTEXT * ctx, int *index_hint)
     mutt_buffer_pool_release (&buf);
     return -1;
   }
-  
+
   /* create .mh_sequences when there isn't one. */
   mutt_buffer_printf (buf, "%s/.mh_sequences", ctx->path);
   if ((i = stat (mutt_b2s (buf), &st_cur)) == -1 && errno == ENOENT)
   {
     char *tmp;
     FILE *fp = NULL;
-    
+
     if (mh_mkstemp (ctx, &fp, &tmp) == 0)
     {
       safe_fclose (&fp);
@@ -2508,6 +2519,33 @@ static int mh_check_mailbox (CONTEXT * ctx, int *index_hint)
 }
 
 
+static int maildir_save_to_header_cache (CONTEXT *ctx, HEADER *h)
+{
+  int rc = 0;
+#if USE_HCACHE
+  header_cache_t *hc;
+
+  hc = mutt_hcache_open (HeaderCache, ctx->path, NULL);
+  rc = mutt_hcache_store (hc, h->path + 3, h, 0, &maildir_hcache_keylen,
+                          MUTT_GENERATE_UIDVALIDITY);
+  mutt_hcache_close (hc);
+#endif
+  return rc;
+}
+
+
+static int mh_save_to_header_cache (CONTEXT *ctx, HEADER *h)
+{
+  int rc = 0;
+#if USE_HCACHE
+  header_cache_t *hc;
+
+  hc = mutt_hcache_open (HeaderCache, ctx->path, NULL);
+  rc = mutt_hcache_store (hc, h->path, h, 0, strlen, MUTT_GENERATE_UIDVALIDITY);
+  mutt_hcache_close (hc);
+#endif
+  return rc;
+}
 
 
 /*
@@ -2517,7 +2555,7 @@ static int mh_check_mailbox (CONTEXT * ctx, int *index_hint)
  */
 
 static FILE *_maildir_open_find_message (const char *folder, const char *unique,
-				  const char *subfolder)
+                                         const char *subfolder)
 {
   BUFFER *dir = mutt_buffer_pool_get ();
   BUFFER *tunique = mutt_buffer_pool_get ();
@@ -2544,7 +2582,7 @@ static FILE *_maildir_open_find_message (const char *folder, const char *unique,
     if (!mutt_strcmp (mutt_b2s (tunique), unique))
     {
       mutt_buffer_printf (fname, "%s/%s/%s", folder, subfolder,
-		de->d_name);
+                          de->d_name);
       fp = fopen (mutt_b2s (fname), "r");	/* __FOPEN_CHECKED__ */
       oe = errno;
       break;
@@ -2573,10 +2611,8 @@ FILE *maildir_open_find_message (const char *folder, const char *msg)
   unique = mutt_buffer_pool_get ();
   maildir_canon_filename (unique, msg);
 
-  if (
-      (fp =
-       _maildir_open_find_message (folder, mutt_b2s (unique),
-				   new_hits > cur_hits ? "new" : "cur"))
+  if ((fp = _maildir_open_find_message (folder, mutt_b2s (unique),
+                                        new_hits > cur_hits ? "new" : "cur"))
       || errno != ENOENT)
   {
     if (new_hits < UINT_MAX && cur_hits < UINT_MAX)
@@ -2587,10 +2623,8 @@ FILE *maildir_open_find_message (const char *folder, const char *msg)
 
     goto cleanup;
   }
-  if (
-      (fp =
-       _maildir_open_find_message (folder, mutt_b2s (unique),
-				   new_hits > cur_hits ? "cur" : "new"))
+  if ((fp = _maildir_open_find_message (folder, mutt_b2s (unique),
+                                        new_hits > cur_hits ? "cur" : "new"))
       || errno != ENOENT)
   {
     if (new_hits < UINT_MAX && cur_hits < UINT_MAX)
@@ -2629,12 +2663,13 @@ int maildir_check_empty (const char *path)
 
   realpath = mutt_buffer_pool_get ();
 
-  do {
+  do
+  {
     /* we do "cur" on the first iteration since its more likely that we'll
      * find old messages without having to scan both subdirs
      */
     mutt_buffer_printf (realpath, "%s/%s", path,
-	      iter == 0 ? "cur" : "new");
+                        iter == 0 ? "cur" : "new");
     if ((dp = opendir (mutt_b2s (realpath))) == NULL)
     {
       r = -1;
@@ -2669,7 +2704,7 @@ int mh_check_empty (const char *path)
   DIR *dp;
   struct dirent *de;
   int r = 1; /* assume empty until we find a message */
-  
+
   if ((dp = opendir (path)) == NULL)
     return -1;
   while ((de = readdir (dp)))
@@ -2681,7 +2716,7 @@ int mh_check_empty (const char *path)
     }
   }
   closedir (dp);
-  
+
   return r;
 }
 
@@ -2754,6 +2789,7 @@ struct mx_ops mx_maildir_ops = {
   .open_new_msg = maildir_open_new_message,
   .check = maildir_check_mailbox,
   .sync = mh_sync_mailbox,
+  .save_to_header_cache = maildir_save_to_header_cache,
 };
 
 struct mx_ops mx_mh_ops = {
@@ -2766,4 +2802,5 @@ struct mx_ops mx_mh_ops = {
   .open_new_msg = mh_open_new_message,
   .check = mh_check_mailbox,
   .sync = mh_sync_mailbox,
+  .save_to_header_cache = mh_save_to_header_cache,
 };
